==> mandrel-23.0.0.0-Final.tar.gz: mandrel-mandrel-23.0.0.0-Final/espresso/src/com.oracle.truffle.espresso/src/com/oracle/truffle/espresso/nodes/quick/invoke/InvokeHandleNode.java <==
/*
 * Copyright (c) 2017, 2020, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 */
package com.oracle.truffle.espresso.nodes.quick.invoke;

import com.oracle.truffle.api.frame.VirtualFrame;
import com.oracle.truffle.espresso.impl.Klass;
import com.oracle.truffle.espresso.impl.Method;
import com.oracle.truffle.espresso.nodes.EspressoFrame;
import com.oracle.truffle.espresso.nodes.methodhandle.MethodHandleIntrinsicNode;
import com.oracle.truffle.espresso.runtime.StaticObject;

public final class InvokeHandleNode extends InvokeQuickNode {

    @Child private MethodHandleIntrinsicNode intrinsic;
    private final boolean hasReceiver;
    private final int argCount;

    public InvokeHandleNode(Method method, Klass accessingKlass, int top, int curBCI) {
        super(method, top, curBCI);
        this.hasReceiver = !method.isStatic();
        this.intrinsic = insert(method.spawnIntrinsicNode(getLanguage(), getContext().getMeta(), accessingKlass, method.getName(), method.getRawSignature()));
        this.argCount = method.getParameterCount() + (method.isStatic() ? 0 : 1) + (method.isInvokeIntrinsic() ? 1 : 0);
    }

    @Override
    protected Object[] getArguments(VirtualFrame frame) {
        Object[] args = new Object[argCount];
        EspressoFrame.popBasicArgumentsWithArray(frame, top, method.getMethod().getParsedSignature(), hasReceiver, args);
        return args;
    }

    @Override
    public int execute(VirtualFrame frame) {
        Object[] args = getArguments(frame);
        if (hasReceiver) {
            nullCheck((StaticObject) args[0]);
        }
        return pushResult(frame, intrinsic.processReturnValue(intrinsic.call(args), method.getMethod().getReturnKind()));
    }
}

==> mandrel-23.1.5.0-Final.tar.gz: mandrel-mandrel-23.1.5.0-Final/espresso/src/com.oracle.truffle.espresso/src/com/oracle/truffle/espresso/nodes/quick/invoke/InvokeHandleNode.java <==
/*
 * Copyright (c) 2017, 2020, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 */
package com.oracle.truffle.espresso.nodes.quick.invoke;

import com.oracle.truffle.api.frame.VirtualFrame;
import com.oracle.truffle.espresso.impl.Klass;
import com.oracle.truffle.espresso.impl.Method;
import com.oracle.truffle.espresso.nodes.EspressoFrame;
import com.oracle.truffle.espresso.nodes.methodhandle.MethodHandleIntrinsicNode;
import com.oracle.truffle.espresso.runtime.staticobject.StaticObject;

public final class InvokeHandleNode extends InvokeQuickNode {

    @Child private MethodHandleIntrinsicNode intrinsic;
    private final boolean hasReceiver;
    private final int argCount;

    public InvokeHandleNode(Method method, Klass accessingKlass, int top, int curBCI) {
        super(method, top, curBCI);
        this.hasReceiver = !method.isStatic();
        this.intrinsic = insert(method.spawnIntrinsicNode(getLanguage(), getContext().getMeta(), accessingKlass, method.getName(), method.getRawSignature()));
        this.argCount = method.getParameterCount() + (method.isStatic() ? 0 : 1) + (method.isInvokeIntrinsic() ? 1 : 0);
    }

    @Override
    protected Object[] getArguments(VirtualFrame frame) {
        Object[] args = new Object[argCount];
        EspressoFrame.popBasicArgumentsWithArray(frame, top, method.getMethod().getParsedSignature(), hasReceiver, args);
        return args;
    }

    @Override
    public int execute(VirtualFrame frame) {
        Object[] args = getArguments(frame);
        if (hasReceiver) {
            nullCheck((StaticObject) args[0]);
        }
        return pushResult(frame, intrinsic.processReturnValue(intrinsic.call(args), method.getMethod().getReturnKind()));
    }
}
